@@ -552,10 +552,7 @@ static NTSTATUS cli_list_old_recv(struct tevent_req *req, TALLOC_CTX *mem_ctx,
 			TALLOC_FREE(finfo);
 			return NT_STATUS_NO_MEMORY;
 		}
-		if (finfo->name == NULL) {
-			TALLOC_FREE(finfo);
-			return NT_STATUS_INVALID_NETWORK_RESPONSE;
-		}
+
 		status = is_bad_finfo_name(state->cli, finfo);
 		if (!NT_STATUS_IS_OK(status)) {
 			smbXcli_conn_disconnect(state->cli->conn, status);
@@ -576,7 +573,7 @@ NTSTATUS cli_list_old(struct cli_state *cli, const char *mask,
 	struct tevent_context *ev;
 	struct tevent_req *req;
 	NTSTATUS status = NT_STATUS_NO_MEMORY;
-	struct file_info *finfo;
+	struct file_info *finfo = NULL;
 	size_t i, num_finfo;
 
 	if (smbXcli_conn_has_async_calls(cli->conn)) {
@@ -794,9 +791,8 @@ static void cli_list_trans_done(struct tevent_req *subreq)
 		if (finfo->name == NULL) {
 			DEBUG(1, ("cli_list: Error: unable to parse name from "
 				  "info level %d\n", state->info_level));
-			tevent_req_nterror(req,
-				NT_STATUS_INVALID_NETWORK_RESPONSE);
-			return;
+			ff_eos = true;
+			break;
 		}
 
 		status = is_bad_finfo_name(state->cli, finfo);
@@ -1046,7 +1042,7 @@ NTSTATUS cli_list(struct cli_state *cli, const char *mask, uint16_t attribute,
 	struct tevent_req *req;
 	NTSTATUS status = NT_STATUS_NO_MEMORY;
 	struct file_info *finfo;
-	size_t i, num_finfo;
+	size_t i, num_finfo = 0;
 	uint16_t info_level;
 
 	if (smbXcli_conn_protocol(cli->conn) >= PROTOCOL_SMB2_02) {
