@@ -20,8 +20,8 @@
 
 #include "includes.h"
 #include "smbd/smbd.h"
+#include "system/filesys.h"
 #include "librpc/gen_ndr/xattr.h"
-#include "../lib/crypto/sha256.h"
 #include "auth.h"
 #include "vfs_acl_common.h"
 
@@ -278,6 +278,26 @@ static int connect_acl_xattr(struct vfs_handle_struct *handle,
 	return 0;
 }
 
+static int acl_xattr_unlinkat(vfs_handle_struct *handle,
+                        struct files_struct *dirfsp,
+                        const struct smb_filename *smb_fname,
+                        int flags)
+{
+	int ret;
+
+	if (flags & AT_REMOVEDIR) {
+		ret = rmdir_acl_common(handle,
+				dirfsp,
+				smb_fname);
+	} else {
+		ret = unlink_acl_common(handle,
+				dirfsp,
+				smb_fname,
+				flags);
+	}
+	return ret;
+}
+
 static NTSTATUS acl_xattr_fget_nt_acl(vfs_handle_struct *handle,
 				      files_struct *fsp,
 				      uint32_t security_info,
@@ -316,15 +336,12 @@ static NTSTATUS acl_xattr_fset_nt_acl(vfs_handle_struct *handle,
 
 static struct vfs_fn_pointers vfs_acl_xattr_fns = {
 	.connect_fn = connect_acl_xattr,
-	.rmdir_fn = rmdir_acl_common,
-	.unlink_fn = unlink_acl_common,
+	.unlinkat_fn = acl_xattr_unlinkat,
 	.chmod_fn = chmod_acl_module_common,
 	.fchmod_fn = fchmod_acl_module_common,
 	.fget_nt_acl_fn = acl_xattr_fget_nt_acl,
 	.get_nt_acl_fn = acl_xattr_get_nt_acl,
 	.fset_nt_acl_fn = acl_xattr_fset_nt_acl,
-	.chmod_acl_fn = chmod_acl_acl_module_common,
-	.fchmod_acl_fn = fchmod_acl_acl_module_common,
 	.sys_acl_set_file_fn = sys_acl_set_file_xattr,
 	.sys_acl_set_fd_fn = sys_acl_set_fd_xattr
 };
