@@ -33,15 +33,14 @@
 #include "param/share.h"
 #include "dsdb/samdb/samdb.h"
 #include "param/param.h"
-#include "file_server/file_server.h"
 #include "ntvfs/ntvfs.h"
 #include "lib/cmdline/popt_common.h"
 /*
   open the smb server sockets
 */
-static void smbsrv_task_init(struct task_server *task)
+static NTSTATUS smbsrv_task_init(struct task_server *task)
 {	
-	NTSTATUS status;
+	NTSTATUS status = NT_STATUS_UNSUCCESSFUL;
 
 	task_server_set_title(task, "task[smbsrv]");
 
@@ -73,6 +72,7 @@ static void smbsrv_task_init(struct task_server *task)
 		wcard = iface_list_wildcard(task);
 		if (wcard == NULL) {
 			DEBUG(0,("No wildcard addresses available\n"));
+			status = NT_STATUS_UNSUCCESSFUL;
 			goto failed;
 		}
 		for (i=0; wcard[i]; i++) {
@@ -87,19 +87,22 @@ static void smbsrv_task_init(struct task_server *task)
 	}
 
 	irpc_add_name(task->msg_ctx, "smb_server");
-	return;
+	return NT_STATUS_OK;
 failed:
 	task_server_terminate(task, "Failed to startup smb server task", true);	
+	return status;
 }
 
 /* called at smbd startup - register ourselves as a server service */
 NTSTATUS server_service_smb_init(TALLOC_CTX *ctx)
 {
-	struct service_details details = {
+	static const struct service_details details = {
 		.inhibit_fork_on_accept = true,
-		.inhibit_pre_fork = true
+		.inhibit_pre_fork = true,
+		.task_init = smbsrv_task_init,
+		.post_fork = NULL
 	};
 	ntvfs_init(cmdline_lp_ctx);
 	share_init();
-	return register_server_service(ctx, "smb", smbsrv_task_init, &details);
+	return register_server_service(ctx, "smb", &details);
 }
